from Xml.Xslt import test_harness

SRC1 = """<dummy/>"""

SRC2 = """\
<?xml version="1.0"?>
<foo>
  <a/>
  <b/>
  <c/>
</foo>"""

FUNCS = {}

def NotTested(tester, title):
    tester.startTest(title)
    tester.warning('Not tested')
    tester.testDone()
    return

def ApplyTest(tester, src, stylesheets, expected, topLevelParams=None,
              compareFunc=None):

    source = test_harness.FileInfo(string=src)
    sheets = [ test_harness.FileInfo(string=sheet) for sheet in stylesheets ]
    test_harness.XsltTest(tester, source, sheets, expected,
                          topLevelParams=topLevelParams,
                          compareFunc=compareFunc)
    return


###########################
# EXSLT - Common
###########################

FUNCS['Common'] = []

def TestNodeSet(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:common="http://exslt.org/common"
  version="1.0"
>

  <xsl:output method="text"/>

  <xsl:template match="/">
    <xsl:variable name='spam'>
      <eggs/>
    </xsl:variable>
    <xsl:value-of select='count(common:node-set($spam)/*)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='name(common:node-set($spam)/*)'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("node-set")
    result = ApplyTest(tester, SRC1, [sty], "1 eggs", compareFunc=cmp)
    tester.groupDone()
    return
    
FUNCS['Common'].append(TestNodeSet)


# 4XSLT used to die a horrible death when computing the union of
# two node-sets that were created with exslt:node-set().
def TestNodeSetUnion(tester):
    sty1 = """\
<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0"
xmlns:exslt="http://exslt.org/common"
>
  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">

    <xsl:variable name="rtf1">
      <a/><b/><c/>
    </xsl:variable>

    <xsl:variable name="rtf2">
      <x/><y/><z/>
    </xsl:variable>

    <xsl:variable name="set1" select="$rtf1"/>
    <xsl:variable name="set2" select="$rtf2"/>

    <results>

      <xsl:variable name="rtf-union" select="exslt:node-set($rtf1)|exslt:node-set($rtf2)"/>
      <info>$rtf1</info>
      <xsl:copy-of select="$rtf1"/>
      <info>$rtf2:</info>
      <xsl:copy-of select="$rtf2"/>
      <info>$rtf-union:</info>
      <xsl:copy-of select="$rtf-union"/>

    </results>

  </xsl:template>

</xsl:stylesheet>"""

    sty2 = """\
<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0"
xmlns:exslt="http://exslt.org/common"
exclude-result-prefixes="exslt">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">

    <xsl:variable name="rtf1">
      <a/><b/><c/>
    </xsl:variable>

    <xsl:variable name="rtf2">
      <x/><y/><z/>
    </xsl:variable>

    <xsl:variable name="set1" select="exslt:node-set($rtf1)"/>
    <xsl:variable name="set2" select="exslt:node-set($rtf2)"/>

    <results>

      <xsl:variable name="set-union" select="$set1|$set2"/>
      <info>$set1</info>
      <xsl:copy-of select="$set1"/>
      <info>$set2:</info>
      <xsl:copy-of select="$set2"/>
      <info>$set-union:</info>
      <xsl:copy-of select="$set-union"/>

    </results>

  </xsl:template>

</xsl:stylesheet>"""

    sty3 = """\
<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0"
xmlns:exslt="http://exslt.org/common"
exclude-result-prefixes="exslt">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">

    <xsl:variable name="rtf2">
      <x/><y/><z/>
    </xsl:variable>

    <xsl:variable name="set1" select="/foo/*"/>
    <xsl:variable name="set2" select="exslt:node-set($rtf2)"/>

    <results>

      <xsl:variable name="set-union" select="$set1|$set2"/>
      <info>$set1</info>
      <xsl:copy-of select="$set1"/>
      <info>$set2:</info>
      <xsl:copy-of select="$set2"/>
      <info>$set-union:</info>
      <xsl:copy-of select="$set-union"/>

    </results>

  </xsl:template>

</xsl:stylesheet>"""

    expected_1 = """<?xml version="1.0" encoding="UTF-8"?>
<results xmlns:exslt="http://exslt.org/common">
  <info>$rtf1</info>
  <a/>
  <b/>
  <c/>
  <info>$rtf2:</info>
  <x/>
  <y/>
  <z/>
  <info>$rtf-union:</info>
  <a/>
  <b/>
  <c/>
  <x/>
  <y/>
  <z/>
</results>"""

    expected_2 = """<?xml version="1.0" encoding="UTF-8"?>
<results>
  <info>$set1</info>
  <a/>
  <b/>
  <c/>
  <info>$set2:</info>
  <x/>
  <y/>
  <z/>
  <info>$set-union:</info>
  <a/>
  <b/>
  <c/>
  <x/>
  <y/>
  <z/>
</results>"""

    expected_3 = """<?xml version="1.0" encoding="UTF-8"?>
<results>
  <info>$set1</info>
  <a/>
  <b/>
  <c/>
  <info>$set2:</info>
  <x/>
  <y/>
  <z/>
  <info>$set-union:</info>
  <a/>
  <b/>
  <c/>
  <x/>
  <y/>
  <z/>
</results>"""

    tester.startGroup("node-set unions")
    ApplyTest(tester, SRC2, [sty1], expected_1)
    ApplyTest(tester, SRC2, [sty2], expected_2)
    ApplyTest(tester, SRC2, [sty3], expected_3)
    tester.groupDone()
    return
    
FUNCS['Common'].append(TestNodeSetUnion)

def TestObjectType(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:common="http://exslt.org/common"
  version="1.0"
>

  <xsl:output method="text"/>

  <xsl:template match="/">
    <xsl:variable name='spam'>
      <eggs/>
    </xsl:variable>
    <xsl:value-of select='common:object-type($spam)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='common:object-type(.)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='common:object-type("1")'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='common:object-type(1)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='common:object-type(1=1)'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("object-type")
    ApplyTest(tester, SRC1, [sty], "RTF node-set string number boolean",
              compareFunc=cmp)
    tester.groupDone()
    return
    
FUNCS['Common'].append(TestObjectType)

def TestDocument(tester):
    sty = """\
<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
    xmlns:common="http://exslt.org/common"
    extension-element-prefixes="common">

<xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>
<xsl:param name="filebase"/>

<xsl:template match="/">
  <xsl:apply-templates select="test"/>
</xsl:template>

<xsl:template match="test">
  <testout>
    <xsl:apply-templates select="data"/>
  </testout>
</xsl:template>

<xsl:template match="data">
  <xsl:variable name="file" select="concat($filebase, ., '.xml')"/>
  <datafile>
    <xsl:text>Writing data file </xsl:text>
    <xsl:value-of select="."/>
  </datafile>
  <common:document href="{$file}" method="xml" indent="yes" encoding="ISO-8859-1">
        <datatree>
          <name><xsl:value-of select="."/></name>
          <what>test</what>
        </datatree>
  </common:document>
</xsl:template>

</xsl:stylesheet>"""

    src = """\
<?xml version="1.0" encoding="ISO-8859-1"?>

<test>
  <data>11</data>
  <data>12</data>
  <data>13</data>
  <data>14</data>
  <data>15</data>
</test>
"""
    
    expected = """<?xml version="1.0" encoding="ISO-8859-1"?>
<testout>
  <datafile>Writing data file 11</datafile>
  <datafile>Writing data file 12</datafile>
  <datafile>Writing data file 13</datafile>
  <datafile>Writing data file 14</datafile>
  <datafile>Writing data file 15</datafile>
</testout>"""

    file_expected = """<?xml version="1.0" encoding="ISO-8859-1"?>
<datatree>
  <name>%s</name>
  <what>test</what>
</datatree>"""

    import tempfile, os

    fileName = tempfile.mktemp()
    tester.startGroup("common:document")
    ApplyTest(tester, src, [sty], expected,
              topLevelParams={'filebase' : fileName})
    tester.startTest("Created Document")
    for data in range(11, 16):
        file = '%s%d.xml' % (fileName, data)
        tester.compare(1, os.path.exists(file))
        fileData = open(file,'r').read()
        tester.compare(file_expected % data, fileData)
        if os.path.exists(file):
            os.unlink(file)
    tester.testDone()
    tester.groupDone()
    return
 
FUNCS['Common'].append(TestDocument)

 ###########################
# EXSLT - Sets
###########################
    
L = FUNCS['Sets'] = []

def TestDifference(tester):
    sty = """<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:set="http://exslt.org/sets"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam/>
  <x:eggs/>

  <xsl:variable name='self' select="document('')/*"/>

  <xsl:template match="/">
    <xsl:variable name='result' select='set:difference($self/x:*, $self/x:spam)'/>
    <xsl:value-of select='count($result)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='name($result)'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("difference")
    result = ApplyTest(tester, SRC1, [sty], "1 x:eggs", compareFunc=cmp)
    tester.groupDone()
    return

L.append(TestDifference)


def TestHasSameNode(tester):
    test = "Test set:has-same-node"
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:set="http://exslt.org/sets"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam/>
  <x:eggs/>

  <xsl:variable name='self' select='document("")/*'/>
  <xsl:variable name='empty' select='/parent::*'/>

  <xsl:template match="/">
    <xsl:variable name='r1' select='set:has-same-node($self/x:*, $self/x:spam)'/>
    <xsl:choose>
      <xsl:when test='$r1'>
        <xsl:text>works</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>broken</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text> </xsl:text>
    <xsl:variable name='r2' select='set:has-same-node($self/x:*, $empty)'/>
    <xsl:choose>
      <xsl:when test='$r2'>
        <xsl:text>broken</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>works</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
"""

    tester.startGroup("has-same-node")
    ApplyTest(tester, SRC1, [sty], "works works", compareFunc=cmp)
    tester.groupDone()
    return

L.append(TestHasSameNode)

    
def TestIntersection(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:set="http://exslt.org/sets"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam/>
  <x:eggs/>

  <xsl:variable name='self' select='document("")/*'/>
  <xsl:variable name='empty' select='/parent::*'/>

  <xsl:template match="/">
    <xsl:variable name='result' select='set:intersection($self/x:*, $self/x:spam)'/>
    <xsl:value-of select='count($result)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='name($result)'/>
  </xsl:template>

</xsl:stylesheet>
"""

    tester.startGroup("intersection")
    ApplyTest(tester, SRC1, [sty], "1 x:spam", compareFunc=cmp)
    tester.groupDone()
    return


L.append(TestIntersection)


def TestDistinct(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:set="http://exslt.org/sets"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:node pos='1'>spam</x:node>
  <x:node pos='2'>eggs</x:node>
  <x:node pos='3'>spam</x:node>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
    <xsl:variable name='result' select='set:distinct($self/x:*)'/>
    <xsl:value-of select='count($result)'/>
    <xsl:for-each select='$result'>
      <xsl:text> </xsl:text>
      <xsl:value-of select='@pos'/>
      <xsl:value-of select='text()'/>
    </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>
"""

    tester.startGroup("distinct")
    ApplyTest(tester, SRC1, [sty], "2 1spam 2eggs", compareFunc=cmp)
    tester.groupDone()

L.append(TestDistinct)

    
def TestLeading(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:set="http://exslt.org/sets"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam/>
  <x:eggs/>
  <x:bread/>
  <x:eggs/>
  <x:spam/>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
  <xsl:variable name='result' select='set:leading($self/x:*, "name()=&apos;x:bread&apos;")'/>
    <xsl:value-of select='count($result)'/>
    <xsl:for-each select='$result'>
      <xsl:text> </xsl:text>
      <xsl:value-of select='name()'/>
    </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>
"""

    tester.startGroup("leading")
    ApplyTest(tester, SRC1, [sty], "0", compareFunc=cmp)
    tester.groupDone()

L.append(TestLeading)

    
def TestTrailing(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:set="http://exslt.org/sets"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam/>
  <x:eggs/>
  <x:bread/>
  <x:eggs/>
  <x:spam/>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
  <xsl:variable name='result' select='set:trailing($self/x:*, "name()=&apos;x:bread&apos;")'/>
    <xsl:value-of select='count($result)'/>
    <xsl:for-each select='$result'>
      <xsl:text> </xsl:text>
      <xsl:value-of select='name()'/>
    </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>
"""

    tester.startGroup("trailing")
    ApplyTest(tester, SRC1, [sty], "0", compareFunc=cmp)
    tester.groupDone()
    
L.append(TestTrailing)

###########################
# EXSLT - Math
###########################
    
L = FUNCS['Math'] = []

def TestMax(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:math="http://exslt.org/math"
  xmlns:common="http://exslt.org/common"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam>2</x:spam>
  <x:eggs>12.0</x:eggs>
  <x:bread>100</x:bread>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
    <xsl:variable name='result' select='math:max($self/x:*)'/>
    <xsl:value-of select='common:object-type($result)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='$result'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("max")
    ApplyTest(tester, SRC1, [sty], "number 100", compareFunc=cmp)
    tester.groupDone()

L.append(TestMax)

    
def TestMin(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:math="http://exslt.org/math"
  xmlns:common="http://exslt.org/common"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam>2</x:spam>
  <x:eggs>12.0</x:eggs>
  <x:bread>100</x:bread>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
    <xsl:variable name='result' select='math:min($self/x:*)'/>
    <xsl:value-of select='common:object-type($result)'/>
    <xsl:text> </xsl:text>
    <xsl:value-of select='$result'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("min")
    ApplyTest(tester, SRC1, [sty], "number 2", compareFunc=cmp)
    tester.groupDone()

L.append(TestMin)

    
def TestHighest(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:math="http://exslt.org/math"
  xmlns:common="http://exslt.org/common"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam>2</x:spam>
  <x:bread>100</x:bread>
  <x:eggs>12.0</x:eggs>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
    <xsl:variable name='result' select='math:highest($self/x:*)'/>
    <xsl:value-of select='name($result)'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("highest")
    ApplyTest(tester, SRC1, [sty], "x:bread", compareFunc=cmp)
    tester.groupDone()
    return

L.append(TestHighest)

    
def TestLowest(tester):
    sty = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:math="http://exslt.org/math"
  xmlns:common="http://exslt.org/common"
  xmlns:x="http://uche.ogbuji.net/dummy"
  version="1.0"
>

  <xsl:output method="text"/>

  <x:spam>2</x:spam>
  <x:bread>100</x:bread>
  <x:eggs>12.0</x:eggs>

  <xsl:variable name='self' select='document("")/*'/>

  <xsl:template match="/">
    <xsl:variable name='result' select='math:lowest($self/x:*)'/>
    <xsl:value-of select='name($result)'/>
  </xsl:template>

</xsl:stylesheet>
"""
    tester.startGroup("lowest")
    ApplyTest(tester, SRC1, [sty], "x:spam", compareFunc=cmp)
    tester.groupDone()
    return

L.append(TestLowest)

###########################
# EXSLT - Functions
###########################
    
L = FUNCS['Functions'] = []

def TestFunction(tester):

    src1 = """<?xml version="1.0"?><foo/>"""

    sty1 = """\
<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:myns="http://stuff.foo"
  xmlns:func="http://exslt.org/functions"
  extension-element-prefixes="func"
  exclude-result-prefixes="myns">

  <xsl:output method="xml"/>

  <func:function name="myns:toUpperCase">
    <xsl:param name="stringToConvert"/>
    <func:result select="translate($stringToConvert,
                         'abcdefghijklmnopqrstuvwxyz',
                         'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </func:function>

  <xsl:template match="/">
    <result><xsl:value-of select="myns:toUpperCase('Hello World')"/></result>
  </xsl:template>

</xsl:stylesheet>"""

    expected1 = """<?xml version="1.0" encoding="UTF-8"?>
<result>HELLO WORLD</result>"""

    sty2 = """\
<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:x="http://uche.ogbuji.net/dummy"
  xmlns:func="http://exslt.org/functions"
  extension-element-prefixes="func"
  version="1.0"
>

  <func:function name="x:count-elements">
    <xsl:for-each select="(//*)[1]">
      <func:result select="count(//*)" />
    </xsl:for-each>
  </func:function>

  <xsl:template match="/">
    <out>
      <xsl:value-of select="x:count-elements()" />
    </out>
  </xsl:template>

</xsl:stylesheet>
"""

    src2 = """\
<doc>
                              <section index="section1"
                                       index2="atr2val">
                                 <section index="subSection1.1">
                                    <p>Hello</p>
                                    <p>Hello again.</p>
                                 </section>
                              </section>
                              <section index="section2">
                                 <p>Hello2</p>
                                 <section index="subSection2.1">
                                    <p>Hello</p>
                                    <p>Hello again.</p>
                                 </section>
                                 <section index="subSection2.2">
                                    <p>Hello</p>
                                    <p>Hello again.</p>
                                 </section>
                                 <p>Hello2 again.</p>
                                 <section index="subSection2.3">
                                    <p>Hello</p>
                                    <p>Hello again.</p>
                                 </section>
                              </section>
                           </doc>
                           """
    expected2 = """<?xml version="1.0" encoding="UTF-8"?>
<out xmlns:x="http://uche.ogbuji.net/dummy">17</out>"""
    tester.startGroup("func:function")
    ApplyTest(tester, src1, [sty1], expected1)
    ApplyTest(tester, src2, [sty2], expected2)
    tester.groupDone()
    return
 
L.append(TestFunction)

###########################
# EXSLT - Date Time
###########################

DATETIME = ['add',
            'add-duration',
            'date',
            'date-time',
            'day-abbreviation',
            'day-in-month',
            'day-in-week',
            'day-in-year',
            'day-name',
            'day-of-week-in-month',
            'difference',
            'duration',
            'format-date',
            'hour-in-day',
            'leap-year',
            'minute-in-hour',
            'month-abbreviation',
            'month-in-year',
            'month-name',
            'parse=date',
            'second-in-minute',
            'seconds',
            'sum',
            'time',
            'week-in-month',
            'week-in-year',
            'year',
            ]

L = FUNCS['Dates and Times'] = []

for title in DATETIME:
    dummy = lambda tester, title=title: NotTested(tester, title)
    L.append(dummy)

###########################
# EXSLT - Strings
###########################

STRINGS = ['align',
           'concat',
           'decode-uri',
           'encode-uri',
           'padding',
           'replace',
           'tokenize',
           ]

L = FUNCS['Strings'] = []

for title in STRINGS:
    dummy = lambda tester, title=title: NotTested(tester, title)
    L.append(dummy)

def TestSplit(tester):
    sty1 = """<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:str="http://exslt.org/strings">

  <xsl:output method="xml" indent="yes"/>

  <xsl:template match="/">
    <result>
      <one>
        <xsl:copy-of select="str:split('a, simple, string',', ')"/>
      </one>
      <two>
        <xsl:copy-of select="str:split('date math str')"/>
      </two>
      <three>
        <xsl:copy-of select="str:split('foo','')"/>
      </three>
    </result>
  </xsl:template>

</xsl:stylesheet>"""

    expected_1 = """<?xml version="1.0" encoding="UTF-8"?>
<result xmlns:str="http://exslt.org/strings">
  <one>
    <token>a</token>
    <token>simple</token>
    <token>string</token>
  </one>
  <two>
    <token>date</token>
    <token>math</token>
    <token>str</token>
  </two>
  <three>
    <token>f</token>
    <token>o</token>
    <token>o</token>
  </three>
</result>"""

    tester.startGroup("str:split()")
    ApplyTest(tester, SRC1, [sty1], expected_1)
    tester.groupDone()
    return

L.append(TestSplit)


###########################
# EXSLT - Regular Expressions
###########################
   
REGEXP = ['match',
          'replace',
          'test',
          ]

L = FUNCS['Regular Expressions'] = []

for title in REGEXP:
    dummy = lambda tester, title=title: NotTested(tester, title)
    L.append(dummy)


def Test(tester):
    for (group, tests) in FUNCS.items():
        tester.startGroup('EXSLT - %s' % group)
        for test in tests:
            test(tester)
        tester.groupDone()
    return
