import os
from Ft.Lib import Uri, Uuid
from Ft.Xml import FtMiniDom, InputSource, Domlette, cDomlette
from Ft.Xml.Lib import TreeCompare
from Ft.Xml.Xslt import Processor, XsltException, Error

try:
    from xml.dom.ext.reader import Sax2
except ImportError:
    pass

# Needed to have comparable id's for test results
from Ft.Xml.Xslt import XsltFunctions
g_ctr = 1
g_map = {}
def GenerateId(context, nodeSet=None):
    global g_ctr
    gen_id = XsltFunctions.GenerateId(context, nodeSet)
    if g_map.has_key(gen_id):
        return g_map[gen_id]
    id = 'id%d' % g_ctr
    g_ctr = g_ctr + 1
    g_map[gen_id] = id
    return id

ExtFunctions = {(None, 'generate-id') : GenerateId}

def CheckXInclude(source):
    return source in ['default', 'minidom', 'cDomlette']


def GetReader(tester):
    source = tester.test_data['source']
    if source == 'cDomlette':
        reader = Domlette._Reader(cDomlette.nonvalParse)
    elif source == '4DOM':
        def nonvalParse(isrc):
            reader = Sax2.Reader()
            return reader.fromStream(isrc.stream,isrc.uri)
        reader = Domlette._Reader(nonvalParse)
    elif source == 'minidom':
        reader = Domlette._Reader(FtMiniDom.nonvalParse)
    else:
        raise ValueError('unknown source %r' % source)
    return reader


class FileInfo:
    def __init__(self, uri=None, string=None, baseUri='', validate=0,
                 nsAware=1, xinclude=0):
        self.uri = uri
        self.string = string
        self.baseUri = baseUri
        self.validate = validate
        self.nsAware = nsAware
        self.xinclude = xinclude

        # Same logic that exists in _4xslt.py
        # The processor only deals with URIs, not file paths
        if uri and os.path.isfile(uri):
            self.uri = Uri.OsPathToUri(uri)
        if not (uri or string):
            raise ValueError('Either string or uri are required')

        if string and not baseUri:
            self.baseUri = 'file:' + Uuid.UuidAsString(Uuid.GenerateUuid())
        return

    def toInputSource(self, factory):
        if self.uri:
            source = factory.fromUri(self.uri)
        else:
            source = factory.fromString(self.string, self.baseUri)
        return source


def XsltTest(tester, source, stylesheets, expected, compareFunc=None,
             ignorePis=1, topLevelParams=None, extensionModules=None,
             exceptionCode=None, stylesheetInputFactory=None,
             documentInputFactory=None, stylesheetIncPaths=None,
             title=''):

    # reset the counter for generate-id
    global g_ctr
    g_ctr = 1

    if not title:
        title = 'Source %(source)s' % (tester.test_data)
    tester.startTest(title)

    if source.xinclude and not CheckXInclude(tester.test_data['source']):
        tester.warning('%s reader does not support XInclude' % 
                       tester.test_data['source'])
        tester.testDone()
        return

    # Create the processor 
    processor = Processor.Processor(stylesheetIncPaths)
    processor.registerExtensionModules([__name__])
    if extensionModules:
        processor.registerExtensionModules(extensionModules)

    # Setup document input for the processor
    factory = documentInputFactory or InputSource.DefaultFactory
    reader = GetReader(tester)
    reader.inputSourceFactory = factory
    processor.setDocumentReader(reader)

    # Do the transformation
    try:
        # Add the stylesheets to the processor
        factory = stylesheetInputFactory or InputSource.DefaultFactory
        for stylesheet in stylesheets:
            processor.appendStylesheet(stylesheet.toInputSource(factory))

        result = processor.run(source.toInputSource(factory),
                               ignorePis=ignorePis,
                               topLevelParams=topLevelParams)
    except XsltException, exception:
        if exceptionCode is None:
            # We wanted to compare the processor result
            tester.exception('Unexpected XsltException raised')
        else:
            # Compare the exception result
            tester.compare(exceptionCode, exception.errorCode, stackLevel=2)
    else:
        if exceptionCode is not None:
            # We wanted to compare the exception result
            tester.error("XsltException(%d) not raised" % exceptionCode,
                         stackLevel=2)
        else:
            # Compare the processor result
            tester.compare(expected, result, diff=1,
                           func=compareFunc or TreeCompare.TreeCompare,
                           stackLevel=2)

    tester.testDone()
    return
